@@ -3,9 +3,7 @@ package Publican;
 use utf8;
 use warnings;
 use strict;
-
 use Carp;
-use version;
 use Config::Simple;
 use XML::TreeBuilder;
 use I18N::LangTags::List;
@@ -17,7 +15,7 @@ use Publican::Localise;
 
 use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK $SINGLETON $LOCALISE);
 
-$VERSION = version->new('1.6.3');
+$VERSION = '2.1';
 @ISA     = qw(Exporter AutoLoader);
 
 @EXPORT
@@ -180,7 +178,7 @@ my %PARAMS = (
     },
     'ec_provider' => {
         descr =>
-            maketext('Eclipse plugin provider. Defaults to "Publican-Publican::VERSION"'),
+            maketext('Eclipse plugin provider. Defaults to "Publican-[_1]"', $VERSION),
     },
     edition => {
         descr => maketext(
@@ -293,8 +291,37 @@ my %PARAMS = (
         descr   => maketext('The brew dist to use for building the web rpm.'),
         default => 'docs-5E',
     },
+    web_home => {
+        descr => maketext(
+            'This is a Publican Website home page, not a standard book.'),
+    },
+    web_host => {
+        descr => maketext(
+            'This is a Publican Website host name, used for searches and Sitemap.Be sure to include the full path to your document tree. e.g. if your documentas are in the docs directory: http://www.example.com/docs'
+        ),
+    },
     web_obsoletes =>
         { descr => maketext('Packages to obsolete in web RPM.'), },
+    web_search => {
+        descr => maketext(
+            'Override the default search form for a Publican Website. By default this will use Gogle search and do a site search if web_host is set.'
+        ),
+    },
+    web_name_label => {
+        descr => maketext(
+            'Override the book name, bottom level, menu label on a Publican Website.'
+        ),
+    },
+    web_product_label => {
+        descr => maketext(
+            'Override the product, top level, menu label on a Publican Website.'
+        ),
+    },
+    web_version_label => {
+        descr => maketext(
+            'Override the version, middle level, menu label on a Publican Website. To hide this meu item set this to: UNUSED'
+        ),
+    },
     xml_lang => {
         descr   => maketext('Language in which XML is authored.'),
         default => 'en-US',
@@ -497,7 +524,7 @@ sub _load_config {
             unless defined $config->param('ec_name');
         $config->param( 'ec_id', "org.$product.$docname" )
             unless defined $config->param('ec_id');
-        $config->param( 'ec_provider', "Publican-$Publican::VERSION" )
+        $config->param( 'ec_provider', "Publican-$VERSION" )
             unless defined $config->param('ec_provider');
     }
     $DEBUG = $self->{config}->param('debug') if ( !$DEBUG );
@@ -699,6 +726,10 @@ sub help_config {
                 . maketext( "Default: [_1]", $PARAMS{$param}->{default} )
                 . "\n" )
             if ( defined( $PARAMS{$param}->{default} ) );
+        logger(   "\t\t"
+                . maketext( "Constraint: [_1]", $PARAMS{$param}->{constraint} )
+                . "\n" )
+            if ( defined( $PARAMS{$param}->{constraint} ) );
 
         logger("\n");
     }
