@@ -29,10 +29,9 @@ use List::Util qw(max);
 use Text::Wrap qw(fill $columns);
 use IO::String;
 
-use version;
 use vars qw(@ISA $VERSION @EXPORT @EXPORT_OK);
 
-$VERSION = version->new('0.1');
+$VERSION = '0.2';
 @ISA     = qw(Exporter AutoLoader);
 @EXPORT  = qw(new_tree dtd_string);
 
@@ -174,7 +173,10 @@ sub build {
 
                     # The basic layout is for the web system
                     # but these formats are used differently
-                    if ( $format eq 'html-desktop' ) {
+                    if ( $self->{publican}->param('web_home') ) {
+                        $path = "publish/home/$lang";
+                    }
+                    elsif ( $format eq 'html-desktop' ) {
                         $path = "publish/desktop/$lang";
                     }
                     elsif ( $format eq 'xml' ) {
@@ -323,7 +325,7 @@ sub setup_xml {
 
         my @xml_files = dir_list( "$tmp_dir/$lang/xml_tmp", '*.xml' );
 
-        # copy css for brand and defaule images for non-brand
+        # copy css for brand and default images for non-brand
         if ( $type eq 'brand' ) {
             dircopy( "$lang/css", "$tmp_dir/$lang/xml/css" )
                 if ( -d "$lang/css" );
@@ -342,6 +344,11 @@ sub setup_xml {
             # copy common files
             my $common_content = $self->{publican}->param('common_content');
             my $brand          = $self->{publican}->param('brand');
+
+            if ( $common_content =~ m/\"/ & $common_content !~ m/\s/ ) {
+                $common_content =~ s/\"//g;
+            }
+
             File::Copy::Recursive::rcopy_glob(
                 $common_content . "/common/en-US/*",
                 "$tmp_dir/$lang/xml/Common_Content"
@@ -507,18 +514,49 @@ sub validate_xml {
 
     my $dir = pushd("$tmp_dir/$lang/xml");
 
-    my $parser = XML::LibXML->new(
-        {   pedantic_parser   => 1,
-            suppress_errors   => 0,
-            suppress_warnings => 1
-        }
-    );
-    $parser->expand_xinclude(1);
+    # 1.69 blows up if you pass some paramaters in
+    my $parser;
+    if ( $XML::LibXML::VERSION >= 1.70 ) {
+        $parser = XML::LibXML->new(
+            {   pedantic_parser   => 1,
+                suppress_errors   => 0,
+                suppress_warnings => 1,
+                line_numbers      => 1,
+                expand_xinclude   => 1
+            }
+        );
+    }
+    else {
+        $parser = XML::LibXML->new();
+        $parser->line_numbers(1);
+        $parser->expand_xinclude(1);
+    }
 
     croak( maketext( "Cannot locate main XML file: '[_1]'", "$docname.xml" ) )
         unless ( -f "$docname.xml" );
 
-    my $source = $parser->parse_file("$docname.xml");
+    my $source;
+    eval { $source = $parser->parse_file("$docname.xml"); };
+
+    if ($@) {
+        if ( ref($@) ) {
+
+            # handle a structured error (XML::LibXML::Error object)
+            croak(
+                maketext(
+                    "FATAL ERROR: [_1]:[_2] in [_3] on line [_4]: [_5]",
+                    $@->domain(),
+                    $@->code(),
+                    $@->file(),
+                    $@->line(),
+                    $@->message(),
+                )
+            );
+        }
+        else {
+            croak( maketext( "FATAL ERROR: [_1]", $@ ) );
+        }
+    }
 
 ## TODO should version be a variable?
     my $dtd_type = qq|-//OASIS//DTD DocBook XML V$dtdver//EN|;
@@ -547,6 +585,9 @@ sub validate_xml {
             $dtd_path =~ s/ /%20/g;
             $dtd_path =~ s/\\/\//g;
         }
+        else {
+            $dtd_path = 'file:///D:/Data/temp/Redhat/DTD/docbookx.dtd';
+        }
     }
     my $dtd = XML::LibXML::Dtd->new( $dtd_type, $dtd_path );
 
@@ -656,6 +697,9 @@ sub transform {
 
     mkdir "$tmp_dir/$lang/$format";
 
+    my $toc_path = '../../../..';
+    $toc_path = '.' if ( $self->{publican}->param('web_home') );
+
     if ( $format eq 'html-single' ) {
 
         $dir = pushd("$tmp_dir/$lang/$format");
@@ -664,6 +708,7 @@ sub transform {
         $xslt_opts{'prod.url'} = "'$prod_url'";
         $xslt_opts{'package'} = "'$TAR_NAME-$lang-$RPM_VERSION-$RPM_RELEASE'";
         $xslt_opts{'embedtoc'} = $embedtoc;
+        $xslt_opts{'tocpath'}  = "'$toc_path'";
     }
     elsif ( $format eq 'html-desktop' ) {
         $xsl_file = $common_config . "/xsl/html-single.xsl";
@@ -684,6 +729,7 @@ sub transform {
         $xslt_opts{'prod.url'} = "'$prod_url'";
         $xslt_opts{'package'} = "'$TAR_NAME-$lang-$RPM_VERSION-$RPM_RELEASE'";
         $xslt_opts{'embedtoc'}             = $embedtoc;
+        $xslt_opts{'tocpath'}              = "'$toc_path'";
         $xslt_opts{'chunk.first.sections'} = $chunk_first;
         $xslt_opts{'chunk.section.depth'}  = $chunk_section_depth;
     }
@@ -741,18 +787,19 @@ sub transform {
         my $key = new Win32::TieRegistry( "LMachine\\Software\\Publican",
             { Delimiter => "\\" } );
 
+        my $new_href = 'file:///D:/Data/temp/Redhat/docbook-xsl-1.75.2';
         if ( $key and $key->GetValue("xsl_path") ) {
-            my $new_href = 'file:///' . $key->GetValue("xsl_path");
+            $new_href = 'file:///' . $key->GetValue("xsl_path");
             $new_href =~ s/ /%20/g;
             $new_href =~ s/\\/\//g;
+        }
 
-            my @nodelist = $style_doc->getElementsByTagName('xsl:import');
-            foreach my $node (@nodelist) {
-                my $href = $node->getAttribute('href');
-                debug_msg("changing $defualt_href to $new_href\n");
-                $href =~ s|$defualt_href|$new_href|;
-                $node->setAttribute( 'href', $href );
-            }
+        my @nodelist = $style_doc->getElementsByTagName('xsl:import');
+        foreach my $node (@nodelist) {
+            my $href = $node->getAttribute('href');
+            debug_msg("changing $defualt_href to $new_href\n");
+            $href =~ s|$defualt_href|$new_href|;
+            $node->setAttribute( 'href', $href );
         }
     }
 
@@ -779,7 +826,7 @@ sub transform {
             . "$lang.pdf";
 
         my $fop_command
-            = qq|CLASSPATH=$classpath fop -q -c $common_config/fop/fop.xconf -fo $docname.fo -pdf ../pdf/$pdf_name|;
+            = qq|CLASSPATH="$classpath" fop -q -c $common_config/fop/fop.xconf -fo $docname.fo -pdf ../pdf/$pdf_name|;
 
 ## TODO find out if we need to set classpath on windows and how
         if ( $^O eq 'MSWin32' ) {
@@ -1031,7 +1078,7 @@ sub adjustColumnWidths {
 
 perl_highlight syntax highlighting
 
-Edit perl_XXX.color in pdf.xsl and .perl_XXX in CSS to change highlight colours
+Edit highlight_color template in pdf.xsl and .perl_XXX in CSS to change highlight colours
 
 Returns: Modified input tree, which is DocBook XML.
 
@@ -1047,9 +1094,9 @@ sub highlight {
 
     my $hl = new Syntax::Highlight::Engine::Kate(
         substitutions => {
-            "<"  => "&lt;",
-            ">"  => "&gt;",
-            "&"  => "&amp;",
+            "<" => "&lt;",
+            ">" => "&gt;",
+            "&" => "&amp;",
         },
         format_table => {
             Alert        => [ "<perl_Alert>",        "</perl_Alert>" ],
@@ -1087,17 +1134,16 @@ sub highlight {
 
     my $parser = XML::LibXML->new();
 
+## BUGBUG testing https://bugzilla.redhat.com/show_bug.cgi?id=604255
+##    my $test = $content->get_node(1);
+##    my $in_string = $test->toString();
+##    $in_string =~ s/^<\?xml version="1.0"\?>\n//gm;
+##debug_msg("Highlighting: " . $in_string . "\n") if $language eq 'C++';
+
     $parser->expand_entities(0);
     my $out_string = $hl->highlightText( $content->string_value() );
-## TODO since all the tags are inline we should really make these rules apply to all tags
-## BUGBUG remove nested block tag, insertCallouts nested block tag limitation :(
-    $out_string =~ s/<perl_Comment>\n<\/perl_Comment>/\n/mg;
+
 ##debug_msg("Highlighting: $out_string\n");
-    $out_string =~ s/\n<\/perl_String>/<\/perl_String>\n/mg;
-    $out_string =~ s/\n<\/perl_Keyword>/<\/perl_Keyword>\n/mg;
-    $out_string
-        =~ s/(<perl_Keyword>\.\.\.)\n/$1<\/perl_Keyword>\n<perl_Keyword>/mg;
-##debug_msg("Highlighting2: $out_string\n");
 
     # this gives an XML::LibXML::DocumentFragment
     my $list = $parser->parse_balanced_chunk($out_string);
@@ -1160,18 +1206,33 @@ sub insertCallouts {
             $index++;
             foreach my $child ( $node->childNodes() ) {
                 if ( $child->nodeName() eq 'area' ) {
+                    my $pos = 0;
                     my $col = $child->getAttribute('coords')
                         || carp(
-                        maketext("'area' require a 'coords' attribute.") );
+                        maketext("'area' requires a 'coords' attribute.") );
+                    if( $col =~ m/^(\d+)\s+(\d+)$/ ) {
+                        $col = $1;
+                        $pos = $2;
+                    }
+
                     push( @{ $callout{$col}{lines} }, $index );
+                    $callout{$col}{'pos'} = $pos ;
                 }
             }
         }
         elsif ( $node->nodeName() eq 'area' ) {
             $index++;
+            my $pos = 0;
             my $col = $node->getAttribute('coords')
-                || carp( maketext("'area' require a 'coords' attribute.") );
+                || carp( maketext("'area' requires a 'coords' attribute.") );
+
+            if( $col =~ m/^(\d+)\s+(\d+)$/ ) {
+                $col = $1;
+                $pos = $2;
+            }
+
             push( @{ $callout{$col}{lines} }, $index );
+            $callout{$col}{'pos'} = $pos;
         }
     }
 
@@ -1256,7 +1317,13 @@ sub insertCallouts {
 
         if ( defined( $callout{$count} ) ) {
             chomp($out_string);
-            my $padding = $position - ( $callout{$count}{'length'} || 0 );
+
+            # if the position requested is less than the line length,
+            # use the calculated position instead
+            my $pos = $callout{$count}{'pos'};
+            $pos = $position if($pos < $callout{$count}{'length'});
+
+            my $padding = $pos - ( $callout{$count}{'length'} || 0 );
             $out_string .= " " x $padding;
 
             foreach my $index (
@@ -1356,6 +1423,120 @@ sub package_brand {
     return;
 }
 
+=head2 package_home
+
+Package a book for use as a Publican Website home page.
+
+=cut
+
+sub package_home {
+    my ( $self, $args ) = @_;
+
+    my $binary = delete( $args->{binary} );
+
+    if ( %{$args} ) {
+        croak(
+            maketext(
+                "unknown arguments: [_1]", join( ", ", keys %{$args} )
+            )
+        );
+    }
+
+    my $tmp_dir    = $self->{publican}->param('tmp_dir');
+    my $product    = $self->{publican}->param('product');
+    my $version    = $self->{publican}->param('version');
+    my $docname    = $self->{publican}->param('docname');
+    my $edition    = $self->{publican}->param('edition');
+    my $configfile = $self->{publican}->param('configfile');
+    my $release    = $self->{publican}->param('release');
+    my $xml_lang   = $self->{publican}->param('xml_lang');
+    my $type       = $self->{publican}->param('type');
+
+    my $name_start = "$docname";
+    my $tardir     = "$name_start-web-home-$edition";
+
+    mkpath("$tmp_dir/tar/$tardir");
+    mkpath("$tmp_dir/rpm");
+
+    my $langs     = get_all_langs();
+    my @file_list = ('publican.cfg');
+
+    foreach my $file (@file_list) {
+        rcopy( $file, "$tmp_dir/tar/$tardir/." ) if ( -f $file );
+    }
+
+    foreach my $dir ( split( /,/, $langs ), 'pot' ) {
+        dircopy( "$dir", "$tmp_dir/tar/$tardir/$dir" ) if ( -d $dir );
+    }
+
+    my $dir = pushd("$tmp_dir/tar");
+    finddepth( \&del_unwanted_dirs, $tardir );
+    my @files = dir_list( $tardir, '*' );
+    Archive::Tar->create_archive( "$tardir-$release.tgz", 9, @files );
+    $dir = undef;
+
+    fmove( "$tmp_dir/tar/$tardir-$release.tgz", "$tmp_dir/rpm/." );
+
+    my $common_config = $self->{publican}->param('common_config');
+    my $xsl_file      = $common_config . "/xsl/web-home-spec.xsl";
+    $xsl_file =~ s/"//g;    # windows
+
+    my $license = $self->{publican}->param('license');
+    my $brand   = 'publican-' . lc( $self->{publican}->param('brand') );
+    my $doc_url = $self->{publican}->param('doc_url');
+    my $src_url = $self->{publican}->param('src_url');
+    my $os_ver  = $self->{publican}->param('os_ver');
+    my $search  = $self->{publican}->param('web_search');
+    my $host    = $self->{publican}->param('web_host');
+    my $log     = $self->change_log();
+
+    my %xslt_opts = (
+        'book-title' => $name_start,
+        'brand'      => $brand,
+        'prod'       => $product,
+        'prodver'    => $version,
+        'rpmver'     => $edition,
+        'rpmrel'     => $release,
+        'docname'    => $docname,
+        'license'    => $license,
+        'url'        => $doc_url,
+        'src_url'    => $src_url,
+        'log'        => $log,
+        tmpdir       => $tmp_dir,
+        web_search   => $search,
+        web_host     => $host,
+    );
+
+    logger(
+        "\t" . maketext( "Using XML::LibXSLT on [_1]", $xsl_file ) . "\n" );
+
+    my $parser    = XML::LibXML->new();
+    my $xslt      = XML::LibXSLT->new();
+    my $source    = $parser->parse_file( "$xml_lang/$type" . '_Info.xml' );
+    my $style_doc = $parser->parse_file($xsl_file);
+
+    my $stylesheet = $xslt->parse_stylesheet($style_doc);
+
+    my $results = $stylesheet->transform( $source,
+        XML::LibXSLT::xpath_to_string(%xslt_opts) );
+
+    my $outfile;
+    my $spec_name = "$tmp_dir/rpm/$name_start-web-home.spec";
+
+    open( $outfile, ">:utf8", "$spec_name" )
+        || croak( maketext( "Can't open spec file: [_1]", $@ ) );
+    print( $outfile $stylesheet->output_string($results) );
+    close($outfile);
+
+    $self->build_rpm(
+        {   spec   => "$tmp_dir/rpm/$name_start-web-home.spec",
+            binary => $binary
+        }
+    );
+
+    return;
+}
+
 =head2 build_rpm
 
 Build an srpm for books and brands.
@@ -1492,7 +1673,7 @@ sub package {
                     "Project-Id-Version in [_1] is not a valid release value.",
                     $po_file
                 )
-            ) unless $release;
+            ) unless defined $release;
         }
         else {
             croak(
@@ -1525,6 +1706,20 @@ sub package {
     rmtree("$tmp_dir/tar/$tardir/$lang/Common_Content");
     mkpath("$tmp_dir/rpm");
 
+    dircopy( "$xml_lang/icons", "$tmp_dir/tar/$tardir/icons" )
+        if ( -e "$xml_lang/icons" );
+    dircopy( "$lang/icons", "$tmp_dir/tar/$tardir/icons" )
+        if ( -e "$lang/icons" );
+    finddepth( \&del_unwanted_dirs, "$tmp_dir/tar/$tardir/icons" )
+        if ( -e "$tmp_dir/tar/$tardir/icons" );
+
+    dircopy( "$xml_lang/files", "$tmp_dir/tar/$tardir/$lang/files" )
+        if ( -e "$xml_lang/files" );
+    dircopy( "$lang/files", "$tmp_dir/tar/$tardir/$lang/files" )
+        if ( -e "$lang/files" );
+    finddepth( \&del_unwanted_dirs, "$tmp_dir/tar/$tardir/$lang/files" )
+        if ( -e "$tmp_dir/tar/$tardir/files" );
+
     $self->{publican}->{config}->param( 'xml_lang', $lang );
 
     # Need to remove scm from packaged set to avoid fetching from repo
@@ -1584,6 +1779,7 @@ sub package {
         language      => $language,
         abstract      => $abstract,
         tmpdir        => $tmp_dir,
+        ICONS         => ( -e "$xml_lang/icons" ? 1 : 0 ),
     );
 
     logger(
@@ -1949,6 +2145,8 @@ sub dtd_string {
         my $key = new Win32::TieRegistry( "LMachine\\Software\\Publican",
             { Delimiter => "\\" } );
 
+        $uri = 'file:///D:/Data/temp/Redhat/DTD/docbookx.dtd';
+
         if ( $key and $key->GetValue("dtd_path") ) {
             $uri = 'file:///' . $key->GetValue("dtd_path") . '/docbookx.dtd';
         }
