@@ -4,7 +4,6 @@ use utf8;
 use strict;
 use warnings;
 use Carp;
-use version;
 use Publican;
 use Publican::Builder;
 use File::Copy::Recursive qw(fcopy rcopy dircopy fmove rmove dirmove);
@@ -15,14 +14,14 @@ use Locale::PO;
 
 use vars qw($VERSION);
 
-$VERSION = 0.1;
+$VERSION = '0.2';
 
 # What tags do we translate?
 my $TRANSTAGS
-    = qr/^(?:ackno|bridgehead|caption|conftitle|contrib|entry|firstname|glossterm|indexterm|jobtitle|keyword|label|lastname|lineannotation|lotentry|member|orgdiv|orgname|othername|para|refclass|refdescriptor|refentrytitle|refmiscinfo|refname|refpurpose|releaseinfo|revremark|screeninfo|secondaryie|seealsoie|seeie|seg|segtitle|simpara|subtitle|surname|term|termdef|tertiaryie|title|titleabbrev|screen|programlisting|literallayout)$/;
+    = qr/^(?:ackno|bridgehead|caption|conftitle|contrib|entry|firstname|glossterm|indexterm|jobtitle|keyword|label|lastname|lineannotation|lotentry|member|orgdiv|orgname|othername|para|productname|refclass|refdescriptor|refentrytitle|refmiscinfo|refname|refpurpose|releaseinfo|revremark|screeninfo|secondaryie|seealsoie|seeie|seg|segtitle|simpara|subtitle|surname|term|termdef|tertiaryie|title|titleabbrev|screen|programlisting|literallayout)$/;
 
 # Blocks that contain translatable tags that need to be kept inline
-my $IGNOREBLOCKS = qr/^(?:footnote|citerefentry)$/;
+my $IGNOREBLOCKS = qr/^(?:footnote|citerefentry|indexterm)$/;
 
 # Preserve white space in these tags
 my $VERBATIM = qr/^(?:screen|programlisting|literallayout)$/;
@@ -170,14 +169,14 @@ sub po2xml {
     my $type = $out_doc->attr("_tag");
     my $text = $out_doc->as_XML();
 
-    #    $text =~ s/&#10;//g;
-    #    $text =~ s/&#9;//g;
+##    $text =~ s/&#10;//g;
+##    $text =~ s/&#9;//g;
     $text =~ s/&#38;([a-zA-Z-_0-9]+;)/&$1/g;
     $text =~ s/&#38;/&amp;/g;
-    $text =~ s/&#60;/&lt;/g;
-    $text =~ s/&#62;/&gt;/g;
-    $text =~ s/&#34;/"/g;
-    $text =~ s/&#39;/'/g;
+##    $text =~ s/&#60;/&lt;/g;
+##    $text =~ s/&#62;/&gt;/g;
+##    $text =~ s/&#34;/"/g;
+##    $text =~ s/&#39;/'/g;
     $out_doc->root()->delete();
 
     my $OUTDOC;
@@ -328,22 +327,35 @@ sub get_msgs {
 
     # Break strings up in to translatable blocks
     # some block level tags, $IGNOREBLOCKS, should be treated inline
-    # IF they are inside translatable tags
+    # indexterm is special as it's both translatable and ignorable >_<
     foreach my $child (
         $doc->look_down(
             '_tag',
             qr/$TRANSTAGS/,
             sub {
-                not defined(
-                    $_[0]->look_up(
-                        '_tag',
-                        qr/$IGNOREBLOCKS/,
-                        sub {
-                            defined(
-                                $_[0]->look_up( '_tag', qr/$TRANSTAGS/ ) );
-                        }
-                    )
-                );
+                my $inner = $_[0];
+## an index term NOT in a translatable tag should be translated as a block.
+## An indexterm in a translatable tag should be translated inline
+                if ( $inner->tag() eq 'indexterm' ) {
+                    not defined(
+                        $inner->look_up(
+                            '_tag',
+                            qr/$IGNOREBLOCKS/,
+                            sub {
+                                $_[0]->tag() =~ /$TRANSTAGS/
+                                    && $inner->parent()
+                                    && $inner->parent()->tag()
+                                    =~ /$TRANSTAGS/;
+                            },
+                        )
+                    );
+                }
+                else {
+## Other IGNOREBLOCKS tags are completely ignored for translation structure.
+                    not defined(
+                        $inner->look_up( '_tag', qr/$IGNOREBLOCKS/ ) );
+
+                }
             }
         )
         )
@@ -468,7 +480,7 @@ sub merge_msgs {
         }
         else {
 
-            my $trans_node;
+            my $trans_node = XML::Element->new( $child->tag() );
 
             # have to recurse through children
             # pop off all children
@@ -487,25 +499,28 @@ sub merge_msgs {
                     )
                     )
                 {
-                    if ( !$trans_node->is_empty ) {
+                    if ( $trans_node && !$trans_node->is_empty ) {
                         $self->translate(
                             { node => $trans_node, msgids => $msgids } );
                         $child->push_content( $trans_node->content_list() );
                         $trans_node->delete();
+                        $trans_node = XML::Element->new( $child->tag() );
                     }
                     $self->merge_msgs(
                         { out_doc => $nested, msgids => $msgids } );
                     $child->push_content($nested);
                 }
                 else {
-                    $trans_node = XML::Element->new( $child->tag() );
                     $trans_node->push_content($nested);
-                    $self->translate(
-                        { node => $trans_node, msgids => $msgids } );
-                    $child->push_content( $trans_node->content_list() );
-                    $trans_node->delete();
                 }
             }
+
+            if ( $trans_node && !$trans_node->is_empty ) {
+                $self->translate(
+                    { node => $trans_node, msgids => $msgids } );
+                $child->push_content( $trans_node->content_list() );
+                $trans_node->delete();
+            }
         }
     }
 
@@ -723,18 +738,17 @@ sub normalise {
 
     # Escaped entities :(
 ## TODO dupe code from XmlClean
-    #    $norm =~ s/&#10;//g;
-    #    $norm =~ s/&#9;//g;
-    $norm =~ s/&#38;([a-zA-Z-_0-9]+;)/&$1/g;
+##    $norm =~ s/&#10;//g;
+##    $norm =~ s/&#9;//g;
+##    $norm =~ s/&#38;([a-zA-Z-_0-9]+;)/&$1/g;
     $norm =~ s/&#38;/&amp;/g;
-
-    #    $norm =~ s/&amp;#x200B;/&#x200B;/g;
-    #    $norm =~ s/&#x200B; &#x200B;/ /g;
-    #    $norm =~ s/&#x200B; / /g;
-    $norm =~ s/&#60;/&lt;/g;
-    $norm =~ s/&#62;/&gt;/g;
-    $norm =~ s/&#34;/"/g;
-    $norm =~ s/&#39;/'/g;
+    $norm =~ s/&amp;#x200B;/&#x200B;/g;
+##    $norm =~ s/&#x200B; &#x200B;/ /g;
+##    $norm =~ s/&#x200B; / /g;
+##    $norm =~ s/&#60;/&lt;/g;
+##    $norm =~ s/&#62;/&gt;/g;
+##    $norm =~ s/&#34;/"/g;
+##    $norm =~ s/&#39;/'/g;
 
 ##debug_msg("is utf8 norm: '" . utf8::is_utf8($norm) . "'\n");
 ##debug_msg("norm: $norm\n\n");
