@@ -3,7 +3,6 @@ package Publican::XmlClean;
 use strict;
 use warnings;
 use Carp;
-use version;
 use XML::TreeBuilder;
 use Config::Simple;
 use Publican;
@@ -21,7 +20,7 @@ if ($test_collate) {
 
 use vars qw( $VERSION );
 
-$VERSION = version->new('0.1');
+$VERSION = '0.2';
 
 =head1 NAME
 
@@ -922,7 +921,8 @@ sub my_as_XML {
 
                         if ( $MAP_OUT{$tag}->{'block'} ) {
                             if ( $MAP_OUT{$tag}->{'verbatim'} ) {
-                                push( @xml, "\n" );
+## BZ #604465 don't add trailing newline.
+##                                push( @xml, "\n" );
                             }
                             elsif ($node->parent()
                                 && $MAP_OUT{ $node->parent()->{'_tag'} }
